"""Authentication exceptions for the Conduit auth module."""


class AuthenticationError(Exception):
    """Raised when token validation fails."""


class TokenExpiredError(AuthenticationError):
    """Raised when the JWT has expired."""
